// Implementation of the LMTPoint class.

#include <stdlib.h>
#include <vector>
#include "LMTPoint.h"
#include "LMTEdge.h"

// Coordinate-wise equality.
bool LMTPoint::operator==(const LMTPoint &rhs) const {
  return x == rhs.x && y == rhs.y;
}

// Coordinate-wise inequality.
bool LMTPoint::operator!=(const LMTPoint &rhs) const {
  return x != rhs.x || y != rhs.y;
}

// Lexicographic ordering.
bool LMTPoint::operator<(const LMTPoint &rhs) const {
  return x < rhs.x || ((x == rhs.x) && (y < rhs.y));
}

// Check whether this point is to the right of edge.
bool LMTPoint::isRightOf(const LMTEdge *edge) const {
  return (edge->dest->x - edge->rev->dest->x)*(this->y - edge->rev->dest->y) -
    (this->x - edge->rev->dest->x)*(edge->dest->y - edge->rev->dest->y) < 0;
}

// Check whether this point is to the left of edge.
bool LMTPoint::isLeftOf(const LMTEdge *edge) const {
  return (edge->dest->x - edge->rev->dest->x)*(this->y - edge->rev->dest->y) - 
    (this->x - edge->rev->dest->x)*(edge->dest->y - edge->rev->dest->y) > 0;
}

#ifdef DEBUG
// Debug output for this point.
log4cpp::CategoryStream& operator<<(log4cpp::CategoryStream &os, 
				    const LMTPoint &point) {
  os << "(" << point.x << ", " << point.y << ")";
  return os;
}
#endif //DEBUG
