// Declaration of the LMTPoint class, which represents a
// point in the LMT-Skeleton.
#ifndef _LMTPOINT_H
#define _LMTPOINT_H

#include <cstdlib>
#include <vector>

#ifdef DEBUG
#include <log4cpp/Category.hh>
#endif // DEBUG

#include "LMTTypes.h"
#include "LMTEdge.h"


class LMTPoint {
public:
  // Default constructor.
  LMTPoint() : x(0), y(0) { /* void */}
  // Initialize the coordinates.
  LMTPoint(coordT _x, coordT _y) : x(_x), y(_y), visited(false) { /* void */}
  // Coordinate-wise (in)equality.
  bool operator==(const LMTPoint &rhs) const;
  bool operator!=(const LMTPoint &rhs) const;
  // Lexicographic ordering.
  bool operator<(const LMTPoint &rhs) const;
  // Compare whether this point it to the left of edge.
  bool isLeftOf(const LMTEdge *edge) const;
  // Compare whether this point it to the right of edge.
  bool isRightOf(const LMTEdge *edge) const;
public:
  // The x coordinate of the point.
  coordT x;
  // The y coordinate of the point.
  coordT y;
  // Used for the BFS.
  bool visited;
  // The edges incident to this point, in counter-clockwise order.
  std::vector<LMTEdge> edgeList;
};


#ifdef DEBUG
// Debug output for this point.
log4cpp::CategoryStream& operator<<(log4cpp::CategoryStream &os, 
				    const LMTPoint &point);
#endif //DEBUG

#endif // _LMTPOINT_H
