// Common utility functions.
#include <cmath>
#include "util.h"

// Compute the quadrant p is in wrt the origin.
const int quadrant(const LMTPoint &p, const LMTPoint &origin) {
  if (p.x >= origin.x) {
    if (p.y < origin.y)
      return 0;
    else
      return 1;
  } else {
    if (p.y >= origin.y) 
      return 2;
    else
      return 3;
  }
}

// Check whether q is right of the line defined by p1 and p2.
const bool isRightOf(const LMTPoint &p1, const LMTPoint &p2, 
		     const LMTPoint &q) {
  return (p2.x - p1.x)*(q.y - p1.y) - (q.x - p1.x)*(p2.y - p1.y) < 0;
}

// Check whether q is collinear with the line defined by p1 and p2.
const bool isCollinear(const LMTPoint &p1, const LMTPoint &p2, 
			   const LMTPoint &q) {
  return (p2.x - p1.x)*(q.y - p1.y) - (q.x - p1.x)*(p2.y - p1.y) == 0;
}

// Check whether q is left of the line defined by p1 and p2.
const bool isLeftOf(const LMTPoint &p1, const LMTPoint &p2, 
		    const LMTPoint &q) {
  return (p2.x - p1.x)*(q.y - p1.y) - (q.x - p1.x)*(p2.y - p1.y) > 0;
}

// Compute the length of line segment p1p2.
const long double lengthOf(const LMTPoint &p1, const LMTPoint &p2) {
  return sqrt((p2.x - p1.x)*(p2.x - p1.x) + (p2.y - p1.y)*(p2.y - p1.y));
}


